/*
 * Copyright (c) 2008 - 2012, Andy Bierman, All Rights Reserved.
 * Copyright (c) 2012 - 2020, YumaWorks, Inc., All Rights Reserved.
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *

*** Generated by yangdump-sdk wildcat

    Combined SIL module
    module dyn-def-hook
    revision 2020-05-29
    namespace http://netconfcentral.org/ns/dyn-def-hook
    Created: 2020-06-02T22:12:39Z
    CLI parameters:
        format c
        indent 4
        log-level debug3
        module dyn-def-hook
        output dyn-def-hook.c
        sil-edit2
        sil-get2
        unified true

 */

#include <xmlstring.h>

#include "procdefs.h"
#include "agt.h"
#include "agt_action.h"
#include "agt_cb.h"
#include "agt_cfg.h"
#include "agt_rpc.h"
#include "agt_sil_lib.h"
#include "agt_timer.h"
#include "agt_util.h"
#include "agt_val.h"
#include "cfg.h"
#include "dlq.h"
#include "getcb.h"
#include "ncx.h"
#include "ncx_feature.h"
#include "ncxmod.h"
#include "ncxtypes.h"
#include "obj.h"
#include "rpc.h"
#include "status.h"
#include "val.h"
#include "val_child.h"
#include "val_util.h"
#include "xml_val.h"
#include "dyn-def-hook.h"

/* module static variables */
static ncx_module_t *dyn_def_hook_mod;
static obj_template_t *interface_cont_obj;
static val_value_t *interface_cont_val;

/* put your static variables here */

/********************************************************************
* FUNCTION y_dyn_def_hook_init_static_vars
*
* initialize module static variables
*
********************************************************************/
static void y_dyn_def_hook_init_static_vars (void)
{
    dyn_def_hook_mod = NULL;
    interface_cont_obj = NULL;
    interface_cont_val = NULL;

    /* init your static variables here */

} /* y_dyn_def_hook_init_static_vars */


/********************************************************************
* FUNCTION dyn_def_hook_interface_cont_interface_edit
*
* Edit database object callback
* Path: list /interface-cont/interface
* Add object instrumentation in COMMIT phase.
*
* INPUTS:
*     see agt/agt_cb.h for details
*
* RETURNS:
*     error status
********************************************************************/
static status_t
    dyn_def_hook_interface_cont_interface_edit (ses_cb_t *scb,
                                                rpc_msg_t *msg,
                                                agt_cbtyp_t cbtyp,
                                                op_editop_t editop,
                                                val_value_t *newval,
                                                val_value_t *curval)
{
    status_t res = NO_ERR;
    val_value_t *errorval = (curval) ? curval : newval;
    uint32 cur_errs = agt_get_error_count(msg);

    if (LOGDEBUG) {
        log_debug("\nEnter dyn_def_hook_interface_cont_interface_edit callback for %s phase",
            agt_cbtype_name(cbtyp));
    }

    if (editop == OP_EDITOP_MERGE) {
        /* the edit is not really on this node; need to get
         * each child_undo record to get the real edited nodes */
        agt_cfg_transaction_t *txcb = RPC_MSG_TXCB(msg);
        agt_cfg_undo_rec_t *child_edit =
            agt_cfg_first_child_edit(txcb, newval, curval);

        while (child_edit) {
            op_editop_t child_editop = OP_EDITOP_NONE;
            val_value_t *child_newval = NULL;
            val_value_t *child_curval = NULL;

            agt_cfg_child_edit_fields(child_edit,
                &child_editop,
                &child_newval,
                &child_curval);

            if (LOGDEBUG2) {
                log_debug2("\nchild edit %s", op_editop_name(child_editop));
                if (child_newval) {
                    log_debug2_append("\nnew");
                    val_dump_value(child_newval, 0, DBG2);
                }
                if (child_curval) {
                    log_debug2_append("\ncur");
                    val_dump_value(child_curval, 0, DBG2);
                }
            }

            obj_template_t *chobj = (child_newval) ?
                VAL_OBJ(child_newval) : VAL_OBJ(child_curval);
            const xmlChar *name = obj_get_name(chobj);

            /**** process child edits here ****/

            if (!xml_strcmp(name, y_dyn_def_hook_N_type)) {
                /* leaf type (enumeration) */

            } else if (!xml_strcmp(name, y_dyn_def_hook_N_admin_status_def)) {
                /* leaf admin-status-def (enumeration) */

            } else if (!xml_strcmp(name, y_dyn_def_hook_N_admin_status_dyn)) {
                /* leaf admin-status-dyn (enumeration) */

            } else if (!xml_strcmp(name, y_dyn_def_hook_N_ll_leaf)) {
                /* leaf-list ll-leaf (string) */

            }

            child_edit = agt_cfg_next_child_edit(child_edit);
        }
    }

    switch (cbtyp) {
    case AGT_CB_VALIDATE:
        /* description-stmt validation here */
        break;
    case AGT_CB_APPLY:
        /* database manipulation done here */
        break;
    case AGT_CB_COMMIT:
        /* device instrumentation done here */
        switch (editop) {
        case OP_EDITOP_LOAD:
            break;
        case OP_EDITOP_MERGE:
            break;
        case OP_EDITOP_REPLACE:
            break;
        case OP_EDITOP_CREATE:
            break;
        case OP_EDITOP_DELETE:
            break;
        default:
            res = SET_ERROR(ERR_INTERNAL_VAL);
        }
        break;
    case AGT_CB_ROLLBACK:
        /* undo device instrumentation here */
        break;
    default:
        res = SET_ERROR(ERR_INTERNAL_VAL);
    }

    if ((res != NO_ERR) && (cur_errs == agt_get_error_count(msg))) {
        agt_record_error(
            scb,
            &msg->mhdr,
            NCX_LAYER_CONTENT,
            res,
            NULL,
            (errorval) ? NCX_NT_VAL : NCX_NT_NONE,
            errorval,
            (errorval) ? NCX_NT_VAL : NCX_NT_NONE,
            errorval);
    }
    return res;

} /* dyn_def_hook_interface_cont_interface_edit */


/********************************************************************
* FUNCTION dyn_def_hook_interface_cont_edit
*
* Edit database object callback
* Path: container /interface-cont
* Add object instrumentation in COMMIT phase.
*
* INPUTS:
*     see agt/agt_cb.h for details
*
* RETURNS:
*     error status
********************************************************************/
static status_t
    dyn_def_hook_interface_cont_edit (ses_cb_t *scb,
                                      rpc_msg_t *msg,
                                      agt_cbtyp_t cbtyp,
                                      op_editop_t editop,
                                      val_value_t *newval,
                                      val_value_t *curval)
{
    status_t res = NO_ERR;
    val_value_t *errorval = (curval) ? curval : newval;
    uint32 cur_errs = agt_get_error_count(msg);

    if (LOGDEBUG) {
        log_debug("\nEnter dyn_def_hook_interface_cont_edit callback for %s phase",
            agt_cbtype_name(cbtyp));
    }

    if (editop == OP_EDITOP_MERGE) {
        /* the edit is not really on this node; need to get
         * each child_undo record to get the real edited nodes */
        agt_cfg_transaction_t *txcb = RPC_MSG_TXCB(msg);
        agt_cfg_undo_rec_t *child_edit =
            agt_cfg_first_child_edit(txcb, newval, curval);

        while (child_edit) {
            op_editop_t child_editop = OP_EDITOP_NONE;
            val_value_t *child_newval = NULL;
            val_value_t *child_curval = NULL;

            agt_cfg_child_edit_fields(child_edit,
                &child_editop,
                &child_newval,
                &child_curval);

            if (LOGDEBUG2) {
                log_debug2("\nchild edit %s", op_editop_name(child_editop));
                if (child_newval) {
                    log_debug2_append("\nnew");
                    val_dump_value(child_newval, 0, DBG2);
                }
                if (child_curval) {
                    log_debug2_append("\ncur");
                    val_dump_value(child_curval, 0, DBG2);
                }
            }

            obj_template_t *chobj = (child_newval) ?
                VAL_OBJ(child_newval) : VAL_OBJ(child_curval);
            const xmlChar *name = obj_get_name(chobj);

            /**** process child edits here ****/

            if (!xml_strcmp(name, y_dyn_def_hook_N_admin_status_dyn)) {
                /* leaf admin-status-dyn (enumeration) */

            } else if (!xml_strcmp(name, y_dyn_def_hook_N_admin_status_def)) {
                /* leaf admin-status-def (enumeration) */

            }

            child_edit = agt_cfg_next_child_edit(child_edit);
        }
    }

    switch (cbtyp) {
    case AGT_CB_VALIDATE:
        /* description-stmt validation here */
        break;
    case AGT_CB_APPLY:
        /* database manipulation done here */
        break;
    case AGT_CB_COMMIT:
        /* device instrumentation done here */
        switch (editop) {
        case OP_EDITOP_LOAD:
            break;
        case OP_EDITOP_MERGE:
            break;
        case OP_EDITOP_REPLACE:
            break;
        case OP_EDITOP_CREATE:
            break;
        case OP_EDITOP_DELETE:
            break;
        default:
            res = SET_ERROR(ERR_INTERNAL_VAL);
        }

        if (res == NO_ERR) {
            res = agt_check_cache(&interface_cont_val, newval, curval, editop);
        }

        break;
    case AGT_CB_ROLLBACK:
        /* undo device instrumentation here */
        break;
    default:
        res = SET_ERROR(ERR_INTERNAL_VAL);
    }

    if ((res != NO_ERR) && (cur_errs == agt_get_error_count(msg))) {
        agt_record_error(
            scb,
            &msg->mhdr,
            NCX_LAYER_CONTENT,
            res,
            NULL,
            (errorval) ? NCX_NT_VAL : NCX_NT_NONE,
            errorval,
            (errorval) ? NCX_NT_VAL : NCX_NT_NONE,
            errorval);
    }
    return res;

} /* dyn_def_hook_interface_cont_edit */


/********************************************************************
* FUNCTION default_hook_callback
*
* The Dynamic Default Hook function is the user/system callback
* that is invoked before the server checks if the node has
* any defaults and if there is not any YANG defined defaults
* the server can update the node with custom "system" default
* value.
*
* Run an instrumentation-defined function
* for val set default event
*
* INPUTS:
*   parentval == parent val_value node to use
*   obj == object template to use
*
* OUTPUTS
*   *buff == malloced buffer that represents a new default value
*
* RETURNS:
*   status: ERR_NCX_SKIPPED or ERR_NCX_NO_INSTANCE will cause the server
*           to skip over this node. Otherwise the server will report an
*           error and ternimate the transaction
*
*********************************************************************/
static status_t
    default_hook_callback (val_value_t *parentval,
                           obj_template_t *obj,
                           xmlChar **buff)
{
    status_t res = NO_ERR;

    if (LOGDEBUG) {
        log_debug("\n\nEnter Dynamic Default Hook callback for '%s'",
                  obj_get_name(obj));
    }

    /* Check the key value of the parent list to setup the
     * Dynamic leaf accordingly
     */
    val_value_t *child_val = NULL;
    if (parentval) {
        child_val =
            val_find_child(parentval,
                           y_dyn_def_hook_M_dyn_def_hook,
                           (const xmlChar *)"name");
    }

    const xmlChar *defval = NULL;

    /* make sure that this is really the right object that you want to
     * modify. Check if the name of the object is correct and that there
     * is related list key node available.
     */
    if (child_val && !xml_strcmp(obj_get_name(obj), (const xmlChar *)"type")) {
        if (LOGDEBUG) {
            log_debug("\nCurrent list key value:'%s'",
                      VAL_STR(child_val));
        }

        /* 'xml_strncmp' is a String compare for xmlChar for at
         * most 'maxlen' xmlChars, where maxlen is max number of
         * xmlChars to compare
         */
        if (!xml_strncmp(VAL_STR(child_val), (const xmlChar *)"vlan", 4)) {

            /* check if the interface key value is starting with 'vlan' */
            defval = (const xmlChar *)"vlan";
        } else if (!xml_strncmp(VAL_STR(child_val), (const xmlChar *)"lag", 3)) {

            /* check if the interface key value is starting with 'lag' */
            defval = (const xmlChar *)"lag";
        } else {

            /* otherwise just skip over this node */
            return ERR_NCX_NO_INSTANCE;
        }
    } else {

        /* Otherwise, apply some other default handing for all other cases */
        if (typ_is_string(obj_get_basetype(obj))) {
            defval = (const xmlChar *)"NON-DEF-STR";
        } else {
            defval = (const xmlChar *)"non-def-enum";
        }
    }

    if (LOGDEBUG) {
        log_debug("\nDynamic Default value set to:'%s'",
                  defval);
    }

    /* Malloc a buffer that will be used to set the default value
     * NO need to free it here, the server will take care of this buff
     */
    *buff = xml_strdup(defval);
    if (*buff == NULL) {
        return ERR_NCX_SKIPPED;
    }

    return res;

}  /* default_hook_callback */


/********************************************************************
* FUNCTION y_dyn_def_hook_init
*
* initialize the dyn-def-hook server instrumentation library
*
* INPUTS:
*    modname == requested module name
*    revision == requested version (NULL for any)
*
* RETURNS:
*     error status
********************************************************************/
status_t
    y_dyn_def_hook_init (const xmlChar *modname,
                         const xmlChar *revision)
{
    status_t res = NO_ERR;

    y_dyn_def_hook_init_static_vars();

    /* change if custom handling done */
    if (xml_strcmp(modname, y_dyn_def_hook_M_dyn_def_hook)) {
        return ERR_NCX_UNKNOWN_MODULE;
    }

    if (revision && xml_strcmp(revision, y_dyn_def_hook_R_dyn_def_hook)) {
        return ERR_NCX_WRONG_VERSION;
    }

    res =
        ncxmod_load_module(y_dyn_def_hook_M_dyn_def_hook,
                           y_dyn_def_hook_R_dyn_def_hook,
                           agt_get_savedevQ(),
                           &dyn_def_hook_mod);
    if (res != NO_ERR) {
        return res;
    }

    interface_cont_obj =
        ncx_find_object(dyn_def_hook_mod,
                        y_dyn_def_hook_N_interface_cont);
    if (interface_cont_obj == NULL) {
        return ERR_NCX_DEF_NOT_FOUND;
    }

    res =
        agt_cb_register_edit2_callback(y_dyn_def_hook_M_dyn_def_hook,
                                       (const xmlChar *)"/hkdef:interface-cont",
                                       y_dyn_def_hook_R_dyn_def_hook,
                                       dyn_def_hook_interface_cont_edit);
    if (res != NO_ERR) {
        return res;
    }

    res =
        agt_cb_register_edit2_callback(y_dyn_def_hook_M_dyn_def_hook,
                    (const xmlChar *)"/hkdef:interface-cont/hkdef:interface",
                                       y_dyn_def_hook_R_dyn_def_hook,
                                       dyn_def_hook_interface_cont_interface_edit);
    if (res != NO_ERR) {
        return res;
    }

    /* put your module initialization code here */


    /* Register a Dynamic Default Hook callback */
    res =
        agt_cb_def_hook_register(
    (const xmlChar *)"/hkdef:interface-cont/hkdef:admin-status-dyn",
                                 default_hook_callback);
    if (res != NO_ERR) {
        return res;
    }

    res =
        agt_cb_def_hook_register(
    (const xmlChar *)"/hkdef:interface-cont/hkdef:interface/hkdef:type",
                                 default_hook_callback);
    if (res != NO_ERR) {
        return res;
    }

    res =
        agt_cb_def_hook_register(
    (const xmlChar *)"/hkdef:interface-cont/hkdef:interface/hkdef:admin-status-dyn",
                                 default_hook_callback);
    if (res != NO_ERR) {
        return res;
    }

    res =
        agt_cb_def_hook_register(
    (const xmlChar *)"/hkdef:interface-cont/hkdef:interface/hkdef:ll-leaf",
                                 default_hook_callback);
    if (res != NO_ERR) {
        return res;
    }

    return res;

} /* y_dyn_def_hook_init */


/********************************************************************
* FUNCTION y_dyn_def_hook_init2
*
* SIL init phase 2: non-config data structures
* Called after running config is loaded
*
* RETURNS:
*     error status
********************************************************************/
status_t y_dyn_def_hook_init2 (void)
{
    status_t res = NO_ERR;

    interface_cont_val =
        agt_init_cache(y_dyn_def_hook_M_dyn_def_hook,
                       y_dyn_def_hook_N_interface_cont,
                       &res);
    if (res != NO_ERR) {
        return res;
    }

    /* put your init2 code here */

    return res;

} /* y_dyn_def_hook_init2 */


/********************************************************************
* FUNCTION y_dyn_def_hook_cleanup
*    cleanup the server instrumentation library
*
********************************************************************/
void y_dyn_def_hook_cleanup (void)
{

    agt_cb_unregister_callbacks(y_dyn_def_hook_M_dyn_def_hook,
        (const xmlChar *)"/hkdef:interface-cont");

    agt_cb_unregister_callbacks(y_dyn_def_hook_M_dyn_def_hook,
        (const xmlChar *)"/hkdef:interface-cont/hkdef:interface");

    /* put your cleanup code here */

    agt_cb_def_hook_unregister(
            (const xmlChar *)"/hkdef:interface-cont/hkdef:admin-status-dyn");

    agt_cb_def_hook_unregister(
        (const xmlChar *)"/hkdef:interface-cont/hkdef:interface/hkdef:type");

    agt_cb_def_hook_unregister(
            (const xmlChar *)"/hkdef:interface-cont/hkdef:interface/hkdef:admin-status-dyn");

    agt_cb_def_hook_unregister(
        (const xmlChar *)"/hkdef:interface-cont/hkdef:interface/hkdef:ll-leaf");


} /* y_dyn_def_hook_cleanup */

/* END SIL dyn_def_hook.c */
